@@ -1,13 +1,17 @@
+import warnings
+from enum import Enum
+from typing import Set, Union
+
 from django.core.exceptions import ImproperlyConfigured
 
 
-class AppSettings(object):
-    class AuthenticationMethod:
+class AppSettings:
+    class AuthenticationMethod(str, Enum):
         USERNAME = "username"
         EMAIL = "email"
         USERNAME_EMAIL = "username_email"
 
-    class EmailVerificationMethod:
+    class EmailVerificationMethod(str, Enum):
         # After signing up, keep the user account inactive until the email
         # address is verified
         MANDATORY = "mandatory"
@@ -18,6 +22,8 @@ class AppSettings(object):
         NONE = "none"
 
     def __init__(self, prefix):
+        from django.conf import settings
+
         self.prefix = prefix
         # If login is by email, email must be required
         assert (
@@ -43,6 +49,17 @@ class AppSettings(object):
                 raise ImproperlyConfigured(
                     "Invalid combination of ACCOUNT_CHANGE_EMAIL and ACCOUNT_MAX_EMAIL_ADDRESSES"
                 )
+        if hasattr(settings, "ACCOUNT_LOGIN_ATTEMPTS_LIMIT") or hasattr(
+            settings, "ACCOUNT_LOGIN_ATTEMPTS_TIMEOUT"
+        ):
+            warnings.warn(
+                "settings.ACCOUNT_LOGIN_ATTEMPTS_LIMIT/TIMEOUT is deprecated, use: settings.ACCOUNT_RATE_LIMITS['login_failed']"
+            )
+
+        if hasattr(settings, "ACCOUNT_EMAIL_CONFIRMATION_COOLDOWN"):
+            warnings.warn(
+                "settings.ACCOUNT_EMAIL_CONFIRMATION_COOLDOWN is deprecated, use: settings.ACCOUNT_RATE_LIMITS['confirm_email']"
+            )
 
     def _setting(self, name, dflt):
         from allauth.utils import get_setting
@@ -90,14 +107,6 @@ class AppSettings(object):
             "EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL", settings.LOGIN_URL
         )
 
-    @property
-    def EMAIL_CONFIRMATION_COOLDOWN(self):
-        """
-        The cooldown in seconds during which, after an email confirmation has
-        been sent, a second confirmation email will not be sent.
-        """
-        return self._setting("EMAIL_CONFIRMATION_COOLDOWN", 3 * 60)
-
     @property
     def EMAIL_REQUIRED(self):
         """
@@ -116,7 +125,19 @@ class AppSettings(object):
             ret = self.EmailVerificationMethod.MANDATORY
         elif ret is False:
             ret = self.EmailVerificationMethod.OPTIONAL
-        return ret
+        return self.EmailVerificationMethod(ret)
+
+    @property
+    def EMAIL_VERIFICATION_BY_CODE_ENABLED(self):
+        return self._setting("EMAIL_VERIFICATION_BY_CODE_ENABLED", False)
+
+    @property
+    def EMAIL_VERIFICATION_BY_CODE_MAX_ATTEMPTS(self):
+        return self._setting("EMAIL_VERIFICATION_BY_CODE_MAX_ATTEMPTS", 3)
+
+    @property
+    def EMAIL_VERIFICATION_BY_CODE_TIMEOUT(self):
+        return self._setting("EMAIL_VERIFICATION_BY_CODE_TIMEOUT", 15 * 60)
 
     @property
     def MAX_EMAIL_ADDRESSES(self):
@@ -129,7 +150,7 @@ class AppSettings(object):
     @property
     def AUTHENTICATION_METHOD(self):
         ret = self._setting("AUTHENTICATION_METHOD", self.AuthenticationMethod.USERNAME)
-        return ret
+        return self.AuthenticationMethod(ret)
 
     @property
     def EMAIL_MAX_LENGTH(self):
@@ -180,24 +201,46 @@ class AppSettings(object):
 
     @property
     def RATE_LIMITS(self):
-        dflt = {
+        rls = self._setting("RATE_LIMITS", {})
+        if rls is False:
+            return {}
+        attempts_amount = self._setting("LOGIN_ATTEMPTS_LIMIT", 5)
+        attempts_timeout = self._setting("LOGIN_ATTEMPTS_TIMEOUT", 60 * 5)
+        login_failed_rl = None
+        if attempts_amount and attempts_timeout:
+            login_failed_rl = f"10/m/ip,{attempts_amount}/{attempts_timeout}s/key"
+
+        if self.EMAIL_VERIFICATION_BY_CODE_ENABLED:
+            confirm_email_rl = "1/10s/key"
+        else:
+            cooldown = self._setting("EMAIL_CONFIRMATION_COOLDOWN", 3 * 60)
+            confirm_email_rl = None
+            if cooldown:
+                confirm_email_rl = f"1/{cooldown}s/key"
+        ret = {
             # Change password view (for users already logged in)
-            "change_password": "5/m",
+            "change_password": "5/m/user",
             # Email management (e.g. add, remove, change primary)
-            "manage_email": "10/m",
+            "manage_email": "10/m/user",
             # Request a password reset, global rate limit per IP
-            "reset_password": "20/m",
-            # Rate limit measured per individual email address
-            "reset_password_email": "5/m",
-            # Reauthentication for users already logged in)
-            "reauthenticate": "10/m",
+            "reset_password": "20/m/ip,5/m/key",
+            # Reauthentication for users already logged in
+            "reauthenticate": "10/m/user",
             # Password reset (the view the password reset email links to).
-            "reset_password_from_key": "20/m",
+            "reset_password_from_key": "20/m/ip",
             # Signups.
-            "signup": "20/m",
-            # NOTE: Login is already protected via `ACCOUNT_LOGIN_ATTEMPTS_LIMIT`
+            "signup": "20/m/ip",
+            # Logins.
+            "login": "30/m/ip",
+            # Request a login code: key is the email.
+            "request_login_code": "20/m/ip,3/m/key",
+            # Logins.
+            "login_failed": login_failed_rl,
+            # Confirm email
+            "confirm_email": confirm_email_rl,
         }
-        return self._setting("RATE_LIMITS", dflt)
+        ret.update(rls)
+        return ret
 
     @property
     def EMAIL_SUBJECT_PREFIX(self):
@@ -213,6 +256,13 @@ class AppSettings(object):
         """
         return self._setting("SIGNUP_FORM_CLASS", None)
 
+    @property
+    def SIGNUP_FORM_HONEYPOT_FIELD(self):
+        """
+        Honeypot field name. Empty string or ``None`` will disable honeypot behavior.
+        """
+        return self._setting("SIGNUP_FORM_HONEYPOT_FIELD", None)
+
     @property
     def USERNAME_REQUIRED(self):
         """
@@ -319,24 +369,6 @@ class AppSettings(object):
     def FORMS(self):
         return self._setting("FORMS", {})
 
-    @property
-    def LOGIN_ATTEMPTS_LIMIT(self):
-        """
-        Number of failed login attempts. When this number is
-        exceeded, the user is prohibited from logging in for the
-        specified `LOGIN_ATTEMPTS_TIMEOUT`
-        """
-        return self._setting("LOGIN_ATTEMPTS_LIMIT", 5)
-
-    @property
-    def LOGIN_ATTEMPTS_TIMEOUT(self):
-        """
-        Time period from last unsuccessful login attempt, during
-        which the user is prohibited from trying to log in.  Defaults to
-        5 minutes.
-        """
-        return self._setting("LOGIN_ATTEMPTS_TIMEOUT", 60 * 5)
-
     @property
     def EMAIL_CONFIRMATION_HMAC(self):
         return self._setting("EMAIL_CONFIRMATION_HMAC", True)
@@ -386,14 +418,57 @@ class AppSettings(object):
             token_generator = EmailAwarePasswordResetTokenGenerator
         return token_generator
 
+    @property
+    def EMAIL_UNKNOWN_ACCOUNTS(self):
+        return self._setting("EMAIL_UNKNOWN_ACCOUNTS", True)
+
     @property
     def REAUTHENTICATION_TIMEOUT(self):
         return self._setting("REAUTHENTICATION_TIMEOUT", 300)
 
+    @property
+    def EMAIL_NOTIFICATIONS(self):
+        return self._setting("EMAIL_NOTIFICATIONS", False)
+
     @property
     def REAUTHENTICATION_REQUIRED(self):
         return self._setting("REAUTHENTICATION_REQUIRED", False)
 
+    @property
+    def LOGIN_BY_CODE_ENABLED(self):
+        return self._setting("LOGIN_BY_CODE_ENABLED", False)
+
+    @property
+    def LOGIN_BY_CODE_MAX_ATTEMPTS(self):
+        return self._setting("LOGIN_BY_CODE_MAX_ATTEMPTS", 3)
+
+    @property
+    def LOGIN_BY_CODE_TIMEOUT(self):
+        return self._setting("LOGIN_BY_CODE_TIMEOUT", 3 * 60)
+
+    @property
+    def LOGIN_TIMEOUT(self):
+        """
+        The maximum allowed time (in seconds) for a login to go through the
+        various login stages. This limits, for example, the time span that the
+        2FA stage remains available.
+        """
+        return self._setting("LOGIN_TIMEOUT", 15 * 60)
+
+    @property
+    def LOGIN_BY_CODE_REQUIRED(self) -> Union[bool, Set[str]]:
+        """
+        When enabled (in case of ``True``), every user logging in is
+        required to input a login confirmation code sent by email.
+        Alternatively, you can specify a set of authentication methods
+        (``"password"``, ``"mfa"``, or ``"socialaccount"``) for which login
+        codes are required.
+        """
+        value = self._setting("LOGIN_BY_CODE_REQUIRED", False)
+        if isinstance(value, bool):
+            return value
+        return set(value)
+
 
 _app_settings = AppSettings("ACCOUNT_")
 
