@@ -1,5 +1,5 @@
-import functools
 from datetime import timedelta
+from typing import Optional
 
 from django.db import models
 from django.db.models import Q
@@ -37,6 +37,7 @@ class EmailAddressManager(models.Manager):
         """
         assert app_settings.CHANGE_EMAIL
         instance = self.get_new(user)
+        email = email.lower()
         if not instance:
             instance = self.model.objects.create(user=user, email=email)
         else:
@@ -50,8 +51,9 @@ class EmailAddressManager(models.Manager):
         return instance
 
     def add_email(self, request, user, email, confirm=False, signup=False):
+        email = email.lower()
         email_address, created = self.get_or_create(
-            user=user, email__iexact=email, defaults={"email": email}
+            user=user, email=email, defaults={"email": email}
         )
 
         if created and confirm:
@@ -68,11 +70,21 @@ class EmailAddressManager(models.Manager):
         except self.model.DoesNotExist:
             return None
 
+    def get_primary_email(self, user) -> Optional[str]:
+        from allauth.account.utils import user_email
+
+        primary = self.get_primary(user)
+        if primary:
+            email = primary.email
+        else:
+            email = user_email(user)
+        return email
+
     def get_users_for(self, email):
         # this is a list rather than a generator because we probably want to
         # do a len() on it right away
         return [
-            address.user for address in self.filter(verified=True, email__iexact=email)
+            address.user for address in self.filter(verified=True, email=email.lower())
         ]
 
     def fill_cache_for_user(self, user, addresses):
@@ -87,27 +99,24 @@ class EmailAddressManager(models.Manager):
     def get_for_user(self, user, email):
         cache_key = "_emailaddress_cache"
         addresses = getattr(user, cache_key, None)
+        email = email.lower()
         if addresses is None:
-            ret = self.get(user=user, email__iexact=email)
+            ret = self.get(user=user, email=email)
             # To avoid additional lookups when e.g.
             # EmailAddress.set_as_primary() starts touching self.user
             ret.user = user
             return ret
         else:
             for address in addresses:
-                if address.email.lower() == email.lower():
+                if address.email == email:
                     return address
             raise self.model.DoesNotExist()
 
     def is_verified(self, email):
-        return self.filter(email__iexact=email, verified=True).exists()
+        return self.filter(email=email.lower(), verified=True).exists()
 
     def lookup(self, emails):
-        q_list = [Q(email__iexact=e) for e in emails]
-        if not q_list:
-            return self.none()
-        q = functools.reduce(lambda a, b: a | b, q_list)
-        return self.filter(q)
+        return self.filter(email__in=[e.lower() for e in emails])
 
 
 class EmailConfirmationManager(models.Manager):
@@ -115,7 +124,7 @@ class EmailConfirmationManager(models.Manager):
         return self.filter(self.expired_q())
 
     def all_valid(self):
-        return self.exclude(self.expired_q())
+        return self.exclude(self.expired_q()).filter(email_address__verified=False)
 
     def expired_q(self):
         sent_threshold = timezone.now() - timedelta(
