@@ -1,18 +1,23 @@
 import datetime
+import time
+from typing import Dict, Optional
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
+from django.contrib.auth.models import AbstractBaseUser
 from django.core import signing
 from django.db import models
-from django.db.models import Index, Q
+from django.db.models import Q
 from django.db.models.constraints import UniqueConstraint
-from django.db.models.functions import Upper
 from django.utils import timezone
 from django.utils.translation import gettext_lazy as _
 
-from . import app_settings, signals
-from .adapter import get_adapter
-from .managers import EmailAddressManager, EmailConfirmationManager
+from allauth.account import app_settings, signals
+from allauth.account.adapter import get_adapter
+from allauth.account.managers import (
+    EmailAddressManager,
+    EmailConfirmationManager,
+)
 
 
 class EmailAddress(models.Model):
@@ -22,6 +27,7 @@ class EmailAddress(models.Model):
         on_delete=models.CASCADE,
     )
     email = models.EmailField(
+        db_index=True,
         max_length=app_settings.EMAIL_MAX_LENGTH,
         verbose_name=_("email address"),
     )
@@ -34,19 +40,29 @@ class EmailAddress(models.Model):
         verbose_name = _("email address")
         verbose_name_plural = _("email addresses")
         unique_together = [("user", "email")]
+        constraints = [
+            UniqueConstraint(
+                fields=["user", "primary"],
+                name="unique_primary_email",
+                condition=Q(primary=True),
+            )
+        ]
         if app_settings.UNIQUE_EMAIL:
-            constraints = [
+            constraints.append(
                 UniqueConstraint(
                     fields=["email"],
                     name="unique_verified_email",
                     condition=Q(verified=True),
                 )
-            ]
-        indexes = [Index(Upper("email"), name="account_emailaddress_upper")]
+            )
 
     def __str__(self):
         return self.email
 
+    def clean(self):
+        super().clean()
+        self.email = self.email.lower()
+
     def can_set_verified(self):
         if self.verified:
             return True
@@ -54,7 +70,7 @@ class EmailAddress(models.Model):
         if app_settings.UNIQUE_EMAIL:
             conflict = (
                 EmailAddress.objects.exclude(pk=self.pk)
-                .filter(verified=True, email__iexact=self.email)
+                .filter(verified=True, email=self.email)
                 .exists()
             )
         return not conflict
@@ -86,10 +102,8 @@ class EmailAddress(models.Model):
         return True
 
     def send_confirmation(self, request=None, signup=False):
-        if app_settings.EMAIL_CONFIRMATION_HMAC:
-            confirmation = EmailConfirmationHMAC(self)
-        else:
-            confirmation = EmailConfirmation.create(self)
+        model = get_emailconfirmation_model()
+        confirmation = model.create(self)
         confirmation.send(request, signup=signup)
         return confirmation
 
@@ -115,11 +129,6 @@ class EmailConfirmationMixin:
         if not email_address.verified:
             confirmed = get_adapter().confirm_email(request, email_address)
             if confirmed:
-                signals.email_confirmed.send(
-                    sender=self.__class__,
-                    request=request,
-                    email_address=email_address,
-                )
                 return email_address
 
     def send(self, request=None, signup=False):
@@ -156,13 +165,20 @@ class EmailConfirmation(EmailConfirmationMixin, models.Model):
         key = get_adapter().generate_emailconfirmation_key(email_address.email)
         return cls._default_manager.create(email_address=email_address, key=key)
 
+    @classmethod
+    def from_key(cls, key):
+        qs = EmailConfirmation.objects.all_valid()
+        qs = qs.select_related("email_address__user")
+        emailconfirmation = qs.filter(key=key.lower()).first()
+        return emailconfirmation
+
     def key_expired(self):
         expiration_date = self.sent + datetime.timedelta(
             days=app_settings.EMAIL_CONFIRMATION_EXPIRE_DAYS
         )
         return expiration_date <= timezone.now()
 
-    key_expired.boolean = True
+    key_expired.boolean = True  # type: ignore[attr-defined]
 
     def confirm(self, request):
         if not self.key_expired():
@@ -174,10 +190,14 @@ class EmailConfirmation(EmailConfirmationMixin, models.Model):
         self.save()
 
 
-class EmailConfirmationHMAC(EmailConfirmationMixin, object):
+class EmailConfirmationHMAC(EmailConfirmationMixin):
     def __init__(self, email_address):
         self.email_address = email_address
 
+    @classmethod
+    def create(cls, email_address):
+        return EmailConfirmationHMAC(email_address)
+
     @property
     def key(self):
         return signing.dumps(obj=self.email_address.pk, salt=app_settings.SALT)
@@ -196,29 +216,53 @@ class EmailConfirmationHMAC(EmailConfirmationMixin, object):
             ret = None
         return ret
 
+    def key_expired(self):
+        return False
+
 
 class Login:
     """
     Represents a user that is in the process of logging in.
+
+    Keyword arguments:
+
+    signup -- Indicates whether or not sending the
+    email is essential (during signup), or if it can be skipped (e.g. in
+    case email verification is optional and we are only logging in).
     """
 
+    # Optional, because we might be prentending logins to prevent user
+    # enumeration.
+    user: Optional[AbstractBaseUser]
+    email_verification: app_settings.EmailVerificationMethod
+    signal_kwargs: Optional[Dict]
+    signup: bool
+    email: Optional[str]
+    state: Dict
+    initiated_at: float
+    redirect_url: Optional[str]
+
     def __init__(
         self,
         user,
-        email_verification,
-        redirect_url=None,
-        signal_kwargs=None,
-        signup=False,
-        email=None,
-        state=None,
+        email_verification: Optional[app_settings.EmailVerificationMethod] = None,
+        redirect_url: Optional[str] = None,
+        signal_kwargs: Optional[Dict] = None,
+        signup: bool = False,
+        email: Optional[str] = None,
+        state: Optional[Dict] = None,
+        initiated_at: Optional[float] = None,
     ):
         self.user = user
+        if not email_verification:
+            email_verification = app_settings.EMAIL_VERIFICATION
         self.email_verification = email_verification
         self.redirect_url = redirect_url
         self.signal_kwargs = signal_kwargs
         self.signup = signup
         self.email = email
         self.state = {} if state is None else state
+        self.initiated_at = initiated_at if initiated_at else time.time()
 
     def serialize(self):
         from allauth.account.utils import user_pk_to_url_str
@@ -232,34 +276,35 @@ class Login:
                 signal_kwargs["sociallogin"] = sociallogin.serialize()
 
         data = {
-            "user_pk": user_pk_to_url_str(self.user),
+            "user_pk": user_pk_to_url_str(self.user) if self.user else None,
             "email_verification": self.email_verification,
             "signup": self.signup,
             "redirect_url": self.redirect_url,
             "email": self.email,
             "signal_kwargs": signal_kwargs,
             "state": self.state,
+            "initiated_at": self.initiated_at,
         }
         return data
 
     @classmethod
     def deserialize(cls, data):
         from allauth.account.utils import url_str_to_user_pk
-        from allauth.socialaccount.models import SocialLogin
 
-        user = (
-            get_user_model()
-            .objects.filter(pk=url_str_to_user_pk(data["user_pk"]))
-            .first()
-        )
-        if user is None:
-            raise ValueError()
+        user = None
+        user_pk = data["user_pk"]
+        if user_pk is not None:
+            user = (
+                get_user_model().objects.filter(pk=url_str_to_user_pk(user_pk)).first()
+            )
         try:
             # :-( Knowledge of the `socialaccount` is entering the `account` app.
             signal_kwargs = data["signal_kwargs"]
             if signal_kwargs is not None:
                 sociallogin = signal_kwargs.get("sociallogin")
                 if sociallogin is not None:
+                    from allauth.socialaccount.models import SocialLogin
+
                     signal_kwargs = signal_kwargs.copy()
                     signal_kwargs["sociallogin"] = SocialLogin.deserialize(sociallogin)
 
@@ -270,6 +315,21 @@ class Login:
                 signup=data["signup"],
                 signal_kwargs=signal_kwargs,
                 state=data["state"],
+                initiated_at=data["initiated_at"],
             )
         except KeyError:
             raise ValueError()
+
+
+def get_emailconfirmation_model():
+    if app_settings.EMAIL_VERIFICATION_BY_CODE_ENABLED:
+        from allauth.account.internal.flows.email_verification_by_code import (
+            EmailVerificationModel,
+        )
+
+        return EmailVerificationModel
+    elif app_settings.EMAIL_CONFIRMATION_HMAC:
+        model = EmailConfirmationHMAC
+    else:
+        model = EmailConfirmation
+    return model
