@@ -69,7 +69,9 @@ def test_ajax_remove_primary(auth_client, user, settings):
     assert data["location"] == reverse("account_email")
 
 
-def test_remove_secondary(auth_client, user):
+def test_remove_secondary(auth_client, user, settings, mailoutbox):
+    settings.ACCOUNT_EMAIL_NOTIFICATIONS = True
+
     secondary = EmailAddress.objects.create(
         email="secondary@email.org", user=user, verified=False, primary=False
     )
@@ -77,8 +79,11 @@ def test_remove_secondary(auth_client, user):
         reverse("account_email"),
         {"action_remove": "", "email": secondary.email},
     )
-    assert not EmailAddress.objects.filter(email=secondary.pk).exists()
+
+    assert not EmailAddress.objects.filter(pk=secondary.pk).exists()
     assertTemplateUsed(resp, "account/messages/email_deleted.txt")
+    assert len(mailoutbox) == 1
+    assert f"{secondary.email} has been removed" in mailoutbox[0].body
 
 
 def test_set_primary_unverified(auth_client, user):
@@ -222,9 +227,10 @@ def test_delete_email_wipes_user_email(user_factory, client):
     assert user_email(user) == ""
 
 
-def test_change_email(user_factory, client, settings):
+def test_change_email(user_factory, client, settings, mailoutbox):
     settings.ACCOUNT_CHANGE_EMAIL = True
     settings.ACCOUNT_EMAIL_CONFIRMATION_HMAC = True
+    settings.ACCOUNT_EMAIL_NOTIFICATIONS = True
 
     user = user_factory(email_verified=True)
     client.force_login(user)
@@ -234,6 +240,8 @@ def test_change_email(user_factory, client, settings):
         {"action_add": "", "email": "change-to@this.org"},
     )
     assert resp.status_code == 302
+    assert len(mailoutbox) == 1
+    assert mailoutbox[0].subject == "[example.com] Please Confirm Your Email Address"
     new_email = EmailAddress.objects.get(email="change-to@this.org")
     key = EmailConfirmationHMAC(new_email).key
     with patch("allauth.account.signals.email_changed.send") as email_changed_mock:
@@ -245,6 +253,9 @@ def test_change_email(user_factory, client, settings):
     assert new_email.verified
     assert new_email.primary
     assert email_changed_mock.called
+    assert len(mailoutbox) == 2
+    assert mailoutbox[1].subject == "[example.com] Email Changed"
+    assert mailoutbox[1].to == [user.email]
 
 
 def test_add(auth_client, user, settings):
@@ -269,7 +280,10 @@ def test_add_with_reauthentication(auth_client, user, user_password, settings):
     )
     assert not EmailAddress.objects.filter(email="john3@example.org").exists()
     assert resp.status_code == 302
-    assert resp["location"] == reverse("account_reauthenticate") + "?next=%2Femail%2F"
+    assert (
+        resp["location"]
+        == reverse("account_reauthenticate") + "?next=%2Faccounts%2Femail%2F"
+    )
     resp = auth_client.post(resp["location"], {"password": user_password})
     assert EmailAddress.objects.filter(email="john3@example.org").exists()
     assertTemplateUsed(resp, "account/messages/email_confirmation_sent.txt")
@@ -295,15 +309,21 @@ def test_add_not_allowed(
         reverse("account_email"),
         {"action_add": "", "email": email},
     )
-    if prevent_enumeration == "strict":
+    if prevent_enumeration:
         assert resp.status_code == 302
-        EmailAddress.objects.get(
+        email_address = EmailAddress.objects.get(
             email=email,
             user=user,
             verified=False,
             primary=False,
         )
         assertTemplateUsed(resp, "account/messages/email_confirmation_sent.txt")
+        key = EmailConfirmationHMAC(email_address).key
+        resp = auth_client.post(reverse("account_confirm_email", args=[key]))
+        assertTemplateUsed(resp, "account/messages/email_confirmation_failed.txt")
+        assert resp.status_code == 302
+        email_address.refresh_from_db()
+        assert not email_address.verified
     else:
         assert resp.status_code == 200
         assert resp.context["form"].errors == {
@@ -380,3 +400,43 @@ def test_dont_lookup_invalid_email(auth_client, email, did_look_up):
             {"action_remove": "", "email": email},
         )
         assert gfu_mock.called == did_look_up
+
+
+def test_add_requires_reauthentication(settings, auth_client):
+    settings.ACCOUNT_REAUTHENTICATION_REQUIRED = True
+    resp = auth_client.post(
+        reverse("account_email"),
+        {"action_add": "", "email": "john3@example.org"},
+    )
+    assert not EmailAddress.objects.filter(email="john3@example.org").exists()
+    assert resp["location"].startswith(reverse("account_reauthenticate"))
+
+
+def test_remove_requires_reauthentication(auth_client, user, settings):
+    settings.ACCOUNT_REAUTHENTICATION_REQUIRED = True
+    secondary = EmailAddress.objects.create(
+        email="secondary@email.org", user=user, verified=False, primary=False
+    )
+    resp = auth_client.post(
+        reverse("account_email"),
+        {"action_remove": "", "email": secondary.email},
+    )
+    assert resp["location"].startswith(reverse("account_reauthenticate"))
+    assert EmailAddress.objects.filter(pk=secondary.pk).exists()
+
+
+def test_set_primary_requires_reauthentication(auth_client, user, settings):
+    settings.ACCOUNT_REAUTHENTICATION_REQUIRED = True
+    primary = EmailAddress.objects.get(email=user.email)
+    secondary = EmailAddress.objects.create(
+        email="secondary@email.org", user=user, verified=True, primary=False
+    )
+    resp = auth_client.post(
+        reverse("account_email"),
+        {"action_primary": "", "email": secondary.email},
+    )
+    assert resp["location"].startswith(reverse("account_reauthenticate"))
+    primary.refresh_from_db()
+    secondary.refresh_from_db()
+    assert primary.primary
+    assert not secondary.primary
