@@ -1,14 +1,16 @@
 import json
-from unittest.mock import patch
+from unittest.mock import ANY, patch
 
-import django
 from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.core import mail
 from django.test.utils import override_settings
 from django.urls import NoReverseMatch, reverse
 
+from pytest_django.asserts import assertTemplateUsed
+
 from allauth.account import app_settings
+from allauth.account.authentication import AUTHENTICATION_METHODS_SESSION_KEY
 from allauth.account.forms import LoginForm
 from allauth.account.models import EmailAddress
 from allauth.tests import TestCase
@@ -46,6 +48,16 @@ class LoginTests(TestCase):
         self.assertRedirects(
             resp, settings.LOGIN_REDIRECT_URL, fetch_redirect_response=False
         )
+        self.assertEqual(
+            self.client.session[AUTHENTICATION_METHODS_SESSION_KEY],
+            [
+                {
+                    "at": ANY,
+                    "username": "@raymond.penners",
+                    "method": "password",
+                }
+            ],
+        )
 
     def _create_user(self, username="john", password="doe", **kwargs):
         user = get_user_model().objects.create(
@@ -129,23 +141,15 @@ class LoginTests(TestCase):
                     "password": ("doe" if is_valid_attempt else "wrong"),
                 },
             )
-            if django.VERSION >= (4, 1):
-                self.assertFormError(
-                    resp.context["form"],
-                    None,
+            self.assertFormError(
+                resp.context["form"],
+                None,
+                (
                     "Too many failed login attempts. Try again later."
                     if is_locked
-                    else "The username and/or password you specified are not correct.",
-                )
-            else:
-                self.assertFormError(
-                    resp,
-                    "form",
-                    None,
-                    "Too many failed login attempts. Try again later."
-                    if is_locked
-                    else "The username and/or password you specified are not correct.",
-                )
+                    else "The username and/or password you specified are not correct."
+                ),
+            )
 
     @override_settings(
         ACCOUNT_AUTHENTICATION_METHOD=app_settings.AuthenticationMethod.EMAIL,
@@ -173,44 +177,26 @@ class LoginTests(TestCase):
         resp = self.client.post(
             reverse("account_login"), {"login": user.email, "password": "bad"}
         )
-        if django.VERSION >= (4, 1):
-            self.assertFormError(
-                resp.context["form"],
-                None,
-                "The email address and/or password you specified are not correct.",
-            )
-        else:
-            self.assertFormError(
-                resp,
-                "form",
-                None,
-                "The email address and/or password you specified are not correct.",
-            )
-
+        self.assertFormError(
+            resp.context["form"],
+            None,
+            "The email address and/or password you specified are not correct.",
+        )
         resp = self.client.post(
             reverse("account_login"), {"login": user.email, "password": "bad"}
         )
-        if django.VERSION >= (4, 1):
-            self.assertFormError(
-                resp.context["form"],
-                None,
-                "Too many failed login attempts. Try again later.",
-            )
-        else:
-            self.assertFormError(
-                resp,
-                "form",
-                None,
-                "Too many failed login attempts. Try again later.",
-            )
-
+        self.assertFormError(
+            resp.context["form"],
+            None,
+            "Too many failed login attempts. Try again later.",
+        )
         self.client.post(reverse("account_reset_password"), data={"email": user.email})
 
         body = mail.outbox[0].body
         self.assertGreater(body.find("https://"), 0)
 
         # Extract URL for `password_reset_from_key` view and access it
-        url = body[body.find("/password/reset/") :].split()[0]
+        url = body[body.find("/accounts/password/reset/") :].split()[0]
         resp = self.client.get(url)
         # Follow the redirect the actual password reset page with the key
         # hidden.
@@ -259,7 +245,7 @@ class LoginTests(TestCase):
             user=user, email="john@example.org", primary=True, verified=True
         )
         EmailAddress.objects.create(
-            user=user, email="john@example.com", primary=True, verified=False
+            user=user, email="john@example.com", primary=False, verified=False
         )
 
         resp = self.client.post(
@@ -335,5 +321,25 @@ def test_login_password_forgotten_link_present(client, db):
     form = LoginForm()
     assert (
         form.fields["password"].help_text
-        == '<a href="/password/reset/">Forgot your password?</a>'
+        == '<a href="/accounts/password/reset/">Forgot your password?</a>'
     )
+
+
+def test_login_while_authenticated(settings, client, user_factory):
+    settings.ACCOUNT_AUTHENTICATED_LOGIN_REDIRECTS = False
+    user_factory(username="john", email="john@example.org", password="doe")
+    user_factory(username="jane", email="jane@example.org", password="doe")
+    redirect_url = settings.LOGIN_REDIRECT_URL
+
+    resp = client.post(reverse("account_login"), {"login": "john", "password": "doe"})
+    assert resp.status_code == 302
+    assert resp["location"] == redirect_url
+    resp = client.post(reverse("account_login"), {"login": "jane", "password": "doe"})
+    assert resp.status_code == 302
+    assert resp["location"] == redirect_url
+
+
+def test_login_page(client, db):
+    resp = client.get(reverse("account_login"))
+    assert resp.status_code == 200
+    assertTemplateUsed(resp, "account/login.html")
