@@ -29,7 +29,7 @@ from .test_models import UUIDUser
 
 
 test_username_validators = [
-    validators.RegexValidator(regex=r"^[a-c]+$", message="not abc", flags=0)
+    validators.RegexValidator(regex=r"^[a-c]+$", message="not abc")
 ]
 
 
