@@ -3,18 +3,18 @@ from collections import OrderedDict
 from typing import Optional
 
 from django.conf import settings
-from django.contrib import messages
-from django.contrib.auth import get_user_model, update_session_auth_hash
+from django.contrib.auth import REDIRECT_FIELD_NAME, get_user_model
 from django.core.exceptions import FieldDoesNotExist
 from django.db import models
 from django.db.models import Q
 from django.utils.encoding import force_str
-from django.utils.http import base36_to_int, int_to_base36, urlencode
+from django.utils.http import base36_to_int, int_to_base36
 
-from allauth.account import app_settings, signals
+from allauth.account import app_settings
 from allauth.account.adapter import get_adapter
+from allauth.account.internal import flows
 from allauth.account.models import Login
-from allauth.core.exceptions import ImmediateHttpResponse
+from allauth.core.internal import httpkit
 from allauth.utils import (
     get_request_param,
     import_callable,
@@ -22,7 +22,7 @@ from allauth.utils import (
 )
 
 
-def _unicode_ci_compare(s1, s2):
+def _unicode_ci_compare(s1, s2) -> bool:
     """
     Perform case-insensitive comparison of two identifiers, using the
     recommended algorithm from Unicode Technical Report 36, section
@@ -33,18 +33,22 @@ def _unicode_ci_compare(s1, s2):
     return norm_s1 == norm_s2
 
 
-def get_next_redirect_url(request, redirect_field_name="next"):
+def get_next_redirect_url(
+    request, redirect_field_name=REDIRECT_FIELD_NAME
+) -> Optional[str]:
     """
     Returns the next URL to redirect to, if it was explicitly passed
     via the request.
     """
     redirect_to = get_request_param(request, redirect_field_name)
-    if not get_adapter().is_safe_url(redirect_to):
+    if redirect_to and not get_adapter().is_safe_url(redirect_to):
         redirect_to = None
     return redirect_to
 
 
-def get_login_redirect_url(request, url=None, redirect_field_name="next", signup=False):
+def get_login_redirect_url(
+    request, url=None, redirect_field_name=REDIRECT_FIELD_NAME, signup=False
+) -> str:
     ret = url
     if url and callable(url):
         # In order to be able to pass url getters around that depend
@@ -63,22 +67,14 @@ def get_login_redirect_url(request, url=None, redirect_field_name="next", signup
 _user_display_callable = None
 
 
-def logout_on_password_change(request, user):
-    # Since it is the default behavior of Django to invalidate all sessions on
-    # password change, this function actually has to preserve the session when
-    # logout isn't desired.
-    if not app_settings.LOGOUT_ON_PASSWORD_CHANGE:
-        update_session_auth_hash(request, user)
-
-
-def default_user_display(user):
+def default_user_display(user) -> str:
+    ret = ""
     if app_settings.USER_MODEL_USERNAME_FIELD:
-        return getattr(user, app_settings.USER_MODEL_USERNAME_FIELD)
-    else:
-        return force_str(user)
+        ret = getattr(user, app_settings.USER_MODEL_USERNAME_FIELD)
+    return ret or force_str(user) or user._meta.verbose_name
 
 
-def user_display(user):
+def user_display(user) -> str:
     global _user_display_callable
     if not _user_display_callable:
         f = getattr(settings, "ACCOUNT_USER_DISPLAY", default_user_display)
@@ -120,10 +116,15 @@ def user_username(user, *args, commit=False):
 
 
 def user_email(user, *args, commit=False):
-    return user_field(user, app_settings.USER_MODEL_EMAIL_FIELD, *args, commit=commit)
+    if args and args[0]:
+        args = [args[0].lower()]
+    ret = user_field(user, app_settings.USER_MODEL_EMAIL_FIELD, *args, commit=commit)
+    if ret:
+        ret = ret.lower()
+    return ret
 
 
-def has_verified_email(user, email=None):
+def has_verified_email(user, email=None) -> bool:
     from .models import EmailAddress
 
     emailaddress = None
@@ -142,19 +143,12 @@ def has_verified_email(user, email=None):
 def perform_login(
     request,
     user,
-    email_verification,
+    email_verification=None,
     redirect_url=None,
     signal_kwargs=None,
     signup=False,
     email=None,
 ):
-    """
-    Keyword arguments:
-
-    signup -- Indicates whether or not sending the
-    email is essential (during signup), or if it can be skipped (e.g. in
-    case email verification is optional and we are only logging in).
-    """
     login = Login(
         user=user,
         email_verification=email_verification,
@@ -163,86 +157,14 @@ def perform_login(
         signup=signup,
         email=email,
     )
-    return _perform_login(request, login)
-
-
-def _perform_login(request, login):
-    # Local users are stopped due to form validation checking
-    # is_active, yet, adapter methods could toy with is_active in a
-    # `user_signed_up` signal. Furthermore, social users should be
-    # stopped anyway.
-    adapter = get_adapter()
-    hook_kwargs = _get_login_hook_kwargs(login)
-    response = adapter.pre_login(request, login.user, **hook_kwargs)
-    if response:
-        return response
-    return resume_login(request, login)
-
-
-def _get_login_hook_kwargs(login):
-    """
-    TODO: Just break backwards compatibility and pass only `login` to
-    `pre/post_login()`.
-    """
-    return dict(
-        email_verification=login.email_verification,
-        redirect_url=login.redirect_url,
-        signal_kwargs=login.signal_kwargs,
-        signup=login.signup,
-        email=login.email,
-    )
-
-
-def resume_login(request, login):
-    from allauth.account.stages import LoginStageController
-
-    adapter = get_adapter()
-    ctrl = LoginStageController(request, login)
-    try:
-        response = ctrl.handle()
-        if response:
-            return response
-        adapter.login(request, login.user)
-        hook_kwargs = _get_login_hook_kwargs(login)
-        response = adapter.post_login(request, login.user, **hook_kwargs)
-        if response:
-            return response
-    except ImmediateHttpResponse as e:
-        response = e.response
-    return response
-
-
-def unstash_login(request, peek=False):
-    login = None
-    if peek:
-        data = request.session.get("account_login")
-    else:
-        data = request.session.pop("account_login", None)
-    if data is not None:
-        try:
-            login = Login.deserialize(data)
-            request._account_login_accessed = True
-        except ValueError:
-            pass
-    return login
-
-
-def stash_login(request, login):
-    request.session["account_login"] = login.serialize()
-    request._account_login_accessed = True
+    return flows.login.perform_login(request, login)
 
 
 def complete_signup(request, user, email_verification, success_url, signal_kwargs=None):
-    if signal_kwargs is None:
-        signal_kwargs = {}
-    signals.user_signed_up.send(
-        sender=user.__class__, request=request, user=user, **signal_kwargs
-    )
-    return perform_login(
+    return flows.signup.complete_signup(
         request,
-        user,
+        user=user,
         email_verification=email_verification,
-        signup=True,
         redirect_url=success_url,
         signal_kwargs=signal_kwargs,
     )
@@ -269,6 +191,7 @@ def cleanup_email_addresses(request, addresses):
         email = valid_email_or_none(address.email)
         if not email:
             continue
+        address.email = email  # `valid_email_or_none` lower cases
         # ... and non-conflicting ones...
         if (
             app_settings.UNIQUE_EMAIL
@@ -285,14 +208,14 @@ def cleanup_email_addresses(request, addresses):
         ):
             # Email address already exists, and is verified as well.
             continue
-        a = e2a.get(email.lower())
+        a = e2a.get(email)
         if a:
             a.primary = a.primary or address.primary
             a.verified = a.verified or address.verified
         else:
             a = address
             a.verified = a.verified or adapter.is_email_verified(request, a.email)
-            e2a[email.lower()] = a
+            e2a[email] = a
         if a.primary:
             primary_addresses.append(a)
             if a.verified:
@@ -335,12 +258,14 @@ def setup_user_email(request, user, addresses):
     stashed_email = adapter.unstash_verified_email(request)
     if stashed_email:
         priority_addresses.append(
-            EmailAddress(user=user, email=stashed_email, primary=True, verified=True)
+            EmailAddress(
+                user=user, email=stashed_email.lower(), primary=True, verified=True
+            )
         )
     email = user_email(user)
     if email:
         priority_addresses.append(
-            EmailAddress(user=user, email=email, primary=True, verified=False)
+            EmailAddress(user=user, email=email.lower(), primary=True, verified=False)
         )
     addresses, primary = cleanup_email_addresses(
         request, priority_addresses + addresses
@@ -349,72 +274,16 @@ def setup_user_email(request, user, addresses):
         a.user = user
         a.save()
     EmailAddress.objects.fill_cache_for_user(user, addresses)
-    if primary and email and email.lower() != primary.email.lower():
+    if primary and (email or "").lower() != primary.email.lower():
         user_email(user, primary.email)
         user.save()
     return primary
 
 
-def send_email_confirmation(request, user, signup=False, email=None):
-    """
-    Email verification mails are sent:
-    a) Explicitly: when a user signs up
-    b) Implicitly: when a user attempts to log in using an unverified
-    email while EMAIL_VERIFICATION is mandatory.
-
-    Especially in case of b), we want to limit the number of mails
-    sent (consider a user retrying a few times), which is why there is
-    a cooldown period before sending a new mail. This cooldown period
-    can be configured in ACCOUNT_EMAIL_CONFIRMATION_COOLDOWN setting.
-
-    TODO: This code is doing way too much. Looking up EmailAddress, creating
-    if not present, etc. To be refactored.
-    """
-    from .models import EmailAddress
-
-    adapter = get_adapter()
-
-    email_address = None
-    if not email:
-        email = user_email(user)
-    if not email:
-        email_address = (
-            EmailAddress.objects.filter(user=user).order_by("verified", "pk").first()
-        )
-        if email_address:
-            email = email_address.email
-
-    if email:
-        if email_address is None:
-            try:
-                email_address = EmailAddress.objects.get_for_user(user, email)
-            except EmailAddress.DoesNotExist:
-                pass
-        if email_address is not None:
-            if not email_address.verified:
-                send_email = adapter.should_send_confirmation_mail(
-                    request, email_address
-                )
-                if send_email:
-                    email_address.send_confirmation(request, signup=signup)
-            else:
-                send_email = False
-        else:
-            send_email = True
-            email_address = EmailAddress.objects.add_email(
-                request, user, email, signup=signup, confirm=True
-            )
-            assert email_address
-        # At this point, if we were supposed to send an email we have sent it.
-        if send_email:
-            adapter.add_message(
-                request,
-                messages.INFO,
-                "account/messages/email_confirmation_sent.txt",
-                {"email": email, "login": not signup, "signup": signup},
-            )
-    if signup:
-        adapter.stash_user(request, user_pk_to_url_str(user))
+def send_email_confirmation(request, user, signup=False, email=None) -> bool:
+    return flows.email_verification.send_verification_email(
+        request, user, signup=signup, email=email
+    )
 
 
 def sync_user_email_addresses(user):
@@ -428,10 +297,7 @@ def sync_user_email_addresses(user):
     from .models import EmailAddress
 
     email = user_email(user)
-    if (
-        email
-        and not EmailAddress.objects.filter(user=user, email__iexact=email).exists()
-    ):
+    if email and not EmailAddress.objects.filter(user=user, email=email).exists():
         # get_or_create() to gracefully handle races
         EmailAddress.objects.get_or_create(
             user=user, email=email, defaults={"primary": False, "verified": False}
@@ -458,7 +324,9 @@ def filter_users_by_username(*username):
     return ret
 
 
-def filter_users_by_email(email, is_active=None, prefer_verified=False):
+def filter_users_by_email(
+    email: str, is_active: Optional[bool] = None, prefer_verified: bool = False
+):
     """Return list of users by email address
 
     Typically one, at most just a few in length.  First we look through
@@ -474,10 +342,8 @@ def filter_users_by_email(email, is_active=None, prefer_verified=False):
     from .models import EmailAddress
 
     User = get_user_model()
-    mails = EmailAddress.objects.filter(email__iexact=email).prefetch_related("user")
-    if is_active is not None:
-        mails = mails.filter(user__is_active=is_active)
-    mails = list(mails)
+    email = email.lower()
+    mails = list(EmailAddress.objects.filter(email=email).select_related("user"))
     is_verified = False
     if prefer_verified:
         verified_mails = list(filter(lambda e: e.verified, mails))
@@ -489,26 +355,25 @@ def filter_users_by_email(email, is_active=None, prefer_verified=False):
         if _unicode_ci_compare(e.email, email):
             users.append(e.user)
     if app_settings.USER_MODEL_EMAIL_FIELD and not is_verified:
-        q_dict = {app_settings.USER_MODEL_EMAIL_FIELD + "__iexact": email}
+        q_dict = {app_settings.USER_MODEL_EMAIL_FIELD: email}
         user_qs = User.objects.filter(**q_dict)
-        if is_active is not None:
-            user_qs = user_qs.filter(is_active=is_active)
         for user in user_qs.iterator():
             user_email = getattr(user, app_settings.USER_MODEL_EMAIL_FIELD)
             if _unicode_ci_compare(user_email, email):
                 users.append(user)
+    if is_active is not None:
+        users = [u for u in set(users) if u.is_active == is_active]
     return list(set(users))
 
 
 def passthrough_next_redirect_url(request, url, redirect_field_name):
-    assert url.find("?") < 0  # TODO: Handle this case properly
     next_url = get_next_redirect_url(request, redirect_field_name)
     if next_url:
-        url = url + "?" + urlencode({redirect_field_name: next_url})
+        url = httpkit.add_query_params(url, {redirect_field_name: next_url})
     return url
 
 
-def user_pk_to_url_str(user):
+def user_pk_to_url_str(user) -> str:
     """
     This should return a string.
     """
@@ -539,39 +404,3 @@ def url_str_to_user_pk(pk_str):
     else:
         pk = pk_field.to_python(pk_str)
     return pk
-
-
-def assess_unique_email(email) -> Optional[bool]:
-    """
-    True -- email is unique
-    False -- email is already in use
-    None -- email is in use, but we should hide that using email verification.
-    """
-    if not filter_users_by_email(email):
-        # All good.
-        return True
-    elif not app_settings.PREVENT_ENUMERATION:
-        # Fail right away.
-        return False
-    elif (
-        app_settings.EMAIL_VERIFICATION
-        == app_settings.EmailVerificationMethod.MANDATORY
-    ):
-        # In case of mandatory verification and enumeration prevention,
-        # we can avoid creating a new account with the same (unverified)
-        # email address, because we are going to send an email anyway.
-        assert app_settings.PREVENT_ENUMERATION
-        return None
-    elif app_settings.PREVENT_ENUMERATION == "strict":
-        # We're going to be strict on enumeration prevention, and allow for
-        # this email address to pass even though it already exists. In this
-        # scenario, you can signup multiple times using the same email
-        # address resulting in multiple accounts with an unverified email.
-        return True
-    else:
-        assert app_settings.PREVENT_ENUMERATION is True
-        # Conflict. We're supposed to prevent enumeration, but we can't
-        # because that means letting the user in, while emails are required
-        # to be unique. In this case, uniqueness takes precedence over
-        # enumeration prevention.
-        return False
