@@ -1,12 +1,14 @@
+from typing import Any, Dict, List, Optional
+
 from django.conf import settings
 from django.contrib.auth import authenticate, get_user_model
 from django.contrib.sites.shortcuts import get_current_site
-from django.core.exceptions import PermissionDenied
+from django.core.exceptions import ImproperlyConfigured, PermissionDenied
 from django.db import models
-from django.utils.crypto import get_random_string
 from django.utils.translation import gettext_lazy as _
 
 import allauth.app_settings
+from allauth import app_settings as allauth_settings
 from allauth.account.models import EmailAddress
 from allauth.account.utils import (
     filter_users_by_email,
@@ -14,11 +16,16 @@ from allauth.account.utils import (
     setup_user_email,
 )
 from allauth.core import context
-from allauth.socialaccount import signals
+from allauth.socialaccount import app_settings, providers, signals
+from allauth.socialaccount.adapter import get_adapter
+from allauth.socialaccount.internal import statekit
+from allauth.utils import get_request_param
 
-from ..utils import get_request_param
-from . import app_settings, providers
-from .adapter import get_adapter
+
+if not allauth_settings.SOCIALACCOUNT_ENABLED:
+    raise ImproperlyConfigured(
+        "allauth.socialaccount not installed, yet its models are imported."
+    )
 
 
 class SocialAppManager(models.Manager):
@@ -36,7 +43,6 @@ class SocialApp(models.Model):
     provider = models.CharField(
         verbose_name=_("provider"),
         max_length=30,
-        choices=providers.registry.as_choices(),
     )
     # For providers that support subproviders, such as OpenID Connect and SAML,
     # this ID identifies that instance. SocialAccount's originating from app
@@ -69,7 +75,7 @@ class SocialApp(models.Model):
         # a ManyToManyField. Note that Facebook requires an app per domain
         # (unless the domains share a common base name).
         # blank=True allows for disabling apps without removing them
-        sites = models.ManyToManyField("sites.Site", blank=True)
+        sites = models.ManyToManyField("sites.Site", blank=True)  # type: ignore[var-annotated]
 
     class Meta:
         verbose_name = _("social application")
@@ -139,7 +145,7 @@ class SocialAccount(models.Model):
             return provider
         adapter = get_adapter()
         provider = self._provider = adapter.get_provider(
-            request, provider=self.provider
+            request or context.request, provider=self.provider
         )
         return provider
 
@@ -169,10 +175,10 @@ class SocialToken(models.Model):
         verbose_name_plural = _("social application tokens")
 
     def __str__(self):
-        return self.token
+        return "%s (%s)" % (self._meta.verbose_name, self.pk)
 
 
-class SocialLogin(object):
+class SocialLogin:
     """
     Represents a social user that is in the process of being logged
     in. This consists of the following information:
@@ -198,23 +204,49 @@ class SocialLogin(object):
     email addresses retrieved from the provider.
     """
 
-    def __init__(self, user=None, account=None, token=None, email_addresses=[]):
+    account: SocialAccount
+    token: Optional[SocialToken]
+    email_addresses: List[EmailAddress]
+    state: Dict
+    _did_authenticate_by_email: Optional[str]
+
+    def __init__(
+        self,
+        user=None,
+        account: Optional[SocialAccount] = None,
+        token: Optional[SocialToken] = None,
+        email_addresses: Optional[List[EmailAddress]] = None,
+    ):
         if token:
             assert token.account is None or token.account == account
         self.token = token
         self.user = user
-        self.account = account
-        self.email_addresses = email_addresses
+        if account:
+            self.account = account
+        self.email_addresses = email_addresses if email_addresses else []
         self.state = {}
 
-    def connect(self, request, user):
+    def connect(self, request, user) -> None:
         self.user = user
         self.save(request, connect=True)
         signals.social_account_added.send(
             sender=SocialLogin, request=request, sociallogin=self
         )
 
-    def serialize(self):
+        get_adapter().send_notification_mail(
+            "socialaccount/email/account_connected",
+            self.user,
+            context={
+                "account": self.account,
+                "provider": self.account.get_provider(),
+            },
+        )
+
+    @property
+    def is_headless(self) -> bool:
+        return bool(self.state.get("headless"))
+
+    def serialize(self) -> Dict[str, Any]:
         serialize_instance = get_adapter().serialize_instance
         ret = dict(
             account=serialize_instance(self.account),
@@ -227,7 +259,7 @@ class SocialLogin(object):
         return ret
 
     @classmethod
-    def deserialize(cls, data):
+    def deserialize(cls, data: Dict[str, Any]) -> "SocialLogin":
         deserialize_instance = get_adapter().deserialize_instance
         account = deserialize_instance(SocialAccount, data["account"])
         user = deserialize_instance(get_user_model(), data["user"])
@@ -247,7 +279,7 @@ class SocialLogin(object):
         ret.state = data["state"]
         return ret
 
-    def save(self, request, connect=False):
+    def save(self, request, connect: bool = False) -> None:
         """
         Saves a new account. Note that while the account is new,
         the user may be an existing one (when connecting accounts)
@@ -266,7 +298,7 @@ class SocialLogin(object):
             setup_user_email(request, user, self.email_addresses)
 
     @property
-    def is_existing(self):
+    def is_existing(self) -> bool:
         """When `False`, this social login represents a temporary account, not
         yet backed by a database record.
         """
@@ -274,18 +306,15 @@ class SocialLogin(object):
             return False
         return get_user_model().objects.filter(pk=self.user.pk).exists()
 
-    def lookup(self):
+    def lookup(self) -> None:
         """Look up the existing local user account to which this social login
         points, if any.
         """
+        self._did_authenticate_by_email = None
         if not self._lookup_by_socialaccount():
-            provider_id = self.account.get_provider().id
-            if app_settings.EMAIL_AUTHENTICATION or app_settings.PROVIDERS.get(
-                provider_id, {}
-            ).get("EMAIL_AUTHENTICATION", False):
-                self._lookup_by_email()
+            self._lookup_by_email()
 
-    def _lookup_by_socialaccount(self):
+    def _lookup_by_socialaccount(self) -> bool:
         assert not self.is_existing
         try:
             a = SocialAccount.objects.get(
@@ -299,44 +328,65 @@ class SocialLogin(object):
             signals.social_account_updated.send(
                 sender=SocialLogin, request=context.request, sociallogin=self
             )
-            # Update token
-            if app_settings.STORE_TOKENS and self.token:
-                assert not self.token.pk
-                try:
-                    t = SocialToken.objects.get(
-                        account=self.account, app=self.token.app
-                    )
-                    t.token = self.token.token
-                    if self.token.token_secret:
-                        # only update the refresh token if we got one
-                        # many oauth2 providers do not resend the refresh token
-                        t.token_secret = self.token.token_secret
-                    t.expires_at = self.token.expires_at
-                    t.save()
-                    self.token = t
-                except SocialToken.DoesNotExist:
-                    self.token.account = a
-                    self.token.save()
+            self._store_token()
             return True
         except SocialAccount.DoesNotExist:
-            pass
+            return False
 
-    def _lookup_by_email(self):
+    def _store_token(self) -> None:
+        # Update token
+        if not app_settings.STORE_TOKENS or not self.token:
+            return
+        assert not self.token.pk
+        app = self.token.app
+        if app and not app.pk:
+            # If the app is not stored in the db, leave the FK empty.
+            app = None
+        try:
+            t = SocialToken.objects.get(account=self.account, app=app)
+            t.token = self.token.token
+            if self.token.token_secret:
+                # only update the refresh token if we got one
+                # many oauth2 providers do not resend the refresh token
+                t.token_secret = self.token.token_secret
+            t.expires_at = self.token.expires_at
+            t.save()
+            self.token = t
+        except SocialToken.DoesNotExist:
+            self.token.account = self.account
+            self.token.app = app
+            self.token.save()
+
+    def _lookup_by_email(self) -> None:
         emails = [e.email for e in self.email_addresses if e.verified]
         for email in emails:
-            users = filter_users_by_email(email, is_active=True, prefer_verified=True)
+            if not get_adapter().can_authenticate_by_email(self, email):
+                continue
+            users = filter_users_by_email(email, prefer_verified=True)
             if users:
                 self.user = users[0]
-                if app_settings.EMAIL_AUTHENTICATION_AUTO_CONNECT:
-                    self.connect(context.request, self.user)
+                self._did_authenticate_by_email = email
                 return
 
-    def get_redirect_url(self, request):
+    def _accept_login(self, request) -> None:
+        from allauth.socialaccount.internal.flows.email_authentication import (
+            wipe_password,
+        )
+
+        if self._did_authenticate_by_email:
+            wipe_password(request, self.user, self._did_authenticate_by_email)
+            if app_settings.EMAIL_AUTHENTICATION_AUTO_CONNECT:
+                self.connect(context.request, self.user)
+
+    def get_redirect_url(self, request) -> Optional[str]:
         url = self.state.get("next")
         return url
 
     @classmethod
-    def state_from_request(cls, request):
+    def state_from_request(cls, request) -> Dict[str, Any]:
+        """
+        TODO: Deprecated! To be integrated with provider.redirect()
+        """
         state = {}
         next_url = get_next_redirect_url(request)
         if next_url:
@@ -347,24 +397,15 @@ class SocialLogin(object):
         return state
 
     @classmethod
-    def stash_state(cls, request):
-        state = cls.state_from_request(request)
-        verifier = get_random_string(16)
-        request.session["socialaccount_state"] = (state, verifier)
-        return verifier
+    def stash_state(cls, request, state: Optional[Dict[str, Any]] = None) -> str:
+        if state is None:
+            # Only for providers that don't support redirect() yet.
+            state = cls.state_from_request(request)
+        return statekit.stash_state(request, state)
 
     @classmethod
-    def unstash_state(cls, request):
-        if "socialaccount_state" not in request.session:
-            raise PermissionDenied()
-        state, verifier = request.session.pop("socialaccount_state")
-        return state
-
-    @classmethod
-    def verify_and_unstash_state(cls, request, verifier):
-        if "socialaccount_state" not in request.session:
-            raise PermissionDenied()
-        state, verifier2 = request.session.pop("socialaccount_state")
-        if verifier != verifier2:
+    def unstash_state(cls, request) -> Optional[Dict[str, Any]]:
+        state = statekit.unstash_last_state(request)
+        if state is None:
             raise PermissionDenied()
         return state
