@@ -1,5 +1,4 @@
 import json
-import requests
 from datetime import timedelta
 
 from django.http import HttpResponseNotAllowed, HttpResponseRedirect
@@ -8,11 +7,9 @@ from django.utils import timezone
 from django.utils.http import urlencode
 from django.views.decorators.csrf import csrf_exempt
 
-import jwt
-
-from allauth.socialaccount.adapter import get_adapter
+from allauth.account.internal.decorators import login_not_required
+from allauth.socialaccount.internal import jwtkit
 from allauth.socialaccount.models import SocialToken
-from allauth.socialaccount.providers.oauth2.client import OAuth2Error
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
     OAuth2CallbackView,
@@ -22,59 +19,25 @@ from allauth.utils import build_absolute_uri, get_request_param
 
 from .apple_session import get_apple_session
 from .client import AppleOAuth2Client
-from .provider import AppleProvider
 
 
 class AppleOAuth2Adapter(OAuth2Adapter):
     client_class = AppleOAuth2Client
-    provider_id = AppleProvider.id
+    provider_id = "apple"
     access_token_url = "https://appleid.apple.com/auth/token"
     authorize_url = "https://appleid.apple.com/auth/authorize"
     public_key_url = "https://appleid.apple.com/auth/keys"
 
-    def _get_apple_public_key(self, kid):
-        response = requests.get(self.public_key_url)
-        response.raise_for_status()
-        try:
-            data = response.json()
-        except json.JSONDecodeError as e:
-            raise OAuth2Error("Error retrieving apple public key.") from e
-
-        for d in data["keys"]:
-            if d["kid"] == kid:
-                return d
-
-    def get_public_key(self, id_token):
-        """
-        Get the public key which matches the `kid` in the id_token header.
-        """
-        kid = jwt.get_unverified_header(id_token)["kid"]
-        apple_public_key = self._get_apple_public_key(kid=kid)
-
-        public_key = jwt.algorithms.RSAAlgorithm.from_jwk(json.dumps(apple_public_key))
-        return public_key
-
-    def get_client_id(self, provider):
-        app = get_adapter().get_app(request=None, provider=self.provider_id)
-        return [aud.strip() for aud in app.client_id.split(",")]
-
-    def get_verified_identity_data(self, id_token):
-        provider = self.get_provider()
-        allowed_auds = self.get_client_id(provider)
-
-        try:
-            public_key = self.get_public_key(id_token)
-            identity_data = jwt.decode(
-                id_token,
-                public_key,
-                algorithms=["RS256"],
-                audience=allowed_auds,
-                issuer="https://appleid.apple.com",
-            )
-            return identity_data
-
-        except jwt.PyJWTError as e:
-            raise OAuth2Error("Invalid id_token") from e
+    @classmethod
+    def get_verified_identity_data(cls, provider, id_token):
+        data = jwtkit.verify_and_decode(
+            credential=id_token,
+            keys_url=cls.public_key_url,
+            issuer="https://appleid.apple.com",
+            audience=provider.get_auds(),
+            lookup_kid=jwtkit.lookup_kid_jwk,
+        )
+        return data
 
     def parse_token(self, data):
         token = SocialToken(
@@ -88,7 +51,9 @@ class AppleOAuth2Adapter(OAuth2Adapter):
 
         # `user_data` is a big flat dictionary with the parsed JWT claims
         # access_tokens, and user info from the apple post.
-        identity_data = self.get_verified_identity_data(data["id_token"])
+        identity_data = AppleOAuth2Adapter.get_verified_identity_data(
+            self.get_provider(), data["id_token"]
+        )
         token.user_data = {**data, **identity_data}
 
         return token
@@ -114,13 +79,12 @@ class AppleOAuth2Adapter(OAuth2Adapter):
             # so return blank dictionary instead
             return {}
 
-    def get_access_token_data(self, request, app, client):
+    def get_access_token_data(self, request, app, client, pkce_code_verifier=None):
         """We need to gather the info from the apple specific login"""
         apple_session = get_apple_session(request)
 
         # Exchange `code`
         code = get_request_param(request, "code")
-        pkce_code_verifier = request.session.pop("pkce_code_verifier", None)
         access_token_data = client.get_access_token(
             code, pkce_code_verifier=pkce_code_verifier
         )
@@ -138,6 +102,7 @@ class AppleOAuth2Adapter(OAuth2Adapter):
 
 
 @csrf_exempt
+@login_not_required
 def apple_post_callback(request, finish_endpoint_name="apple_finish_callback"):
     """
     Apple uses a `form_post` response type, which due to
