@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 from allauth.socialaccount.providers.auth0.provider import Auth0Provider
 from allauth.socialaccount.tests import OAuth2TestsMixin
 from allauth.tests import MockedResponse, TestCase
@@ -8,13 +7,19 @@ class Auth0Tests(OAuth2TestsMixin, TestCase):
     provider_id = Auth0Provider.id
 
     def get_mocked_response(self):
-        return MockedResponse(200, """
+        return MockedResponse(
+            200,
+            """
             {
                 "picture": "https://secure.gravatar.com/avatar/123",
                 "email": "mr.bob@your.Auth0.server.example.com",
                 "id": 2,
-                "user_id": 2,
+                "sub": 2,
                 "identities": [],
                 "name": "Mr Bob"
             }
-        """)
+        """,
+        )
+
+    def get_expected_to_str(self):
+        return "mr.bob@your.Auth0.server.example.com"
