@@ -1,9 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
-from unittest import mock
-from unittest.mock import patch
-
 from django.test.utils import override_settings
 
 from allauth.socialaccount.models import SocialAccount
@@ -87,50 +81,16 @@ class BitbucketOAuth2Tests(OAuth2TestsMixin, TestCase):
         }
     """  # noqa
 
-    def setUp(self):
-        super(BitbucketOAuth2Tests, self).setUp()
-        self.mocks = {
-            "requests": patch(
-                "allauth.socialaccount.providers.bitbucket_oauth2.views.requests"
-            )
-        }
-        self.patches = {name: mocked.start() for (name, mocked) in self.mocks.items()}
-        self.patches["requests"].get.side_effect = [
+    def get_mocked_response(self):
+        return [
+            MockedResponse(200, self.response_data),
+            MockedResponse(200, self.email_response_data),
             MockedResponse(200, self.response_data),
             MockedResponse(200, self.email_response_data),
         ]
 
-    def tearDown(self):
-        for _, mocked in self.mocks.items():
-            mocked.stop()
-
-    def get_mocked_response(self):
-        return [MockedResponse(200, self.response_data)]
-
-    def test_account_tokens(self, multiple_login=False):
-        if multiple_login:
-            self.patches["requests"].get.side_effect = [
-                MockedResponse(200, self.response_data),
-                MockedResponse(200, self.email_response_data),
-                MockedResponse(200, self.response_data),
-                MockedResponse(200, self.email_response_data),
-            ]
-        super(BitbucketOAuth2Tests, self).test_account_tokens(multiple_login)
-        calls = [
-            mock.call("https://api.bitbucket.org/2.0/user", params=mock.ANY),
-            mock.call("https://api.bitbucket.org/2.0/user/emails", params=mock.ANY),
-        ]
-        if multiple_login:
-            calls.extend(
-                [
-                    mock.call("https://api.bitbucket.org/2.0/user", params=mock.ANY),
-                    mock.call(
-                        "https://api.bitbucket.org/2.0/user/emails",
-                        params=mock.ANY,
-                    ),
-                ]
-            )
-        self.patches["requests"].get.assert_has_calls(calls)
+    def get_expected_to_str(self):
+        return "tutorials"
 
     def test_provider_account(self):
         self.login(self.get_mocked_response())
@@ -138,18 +98,9 @@ class BitbucketOAuth2Tests(OAuth2TestsMixin, TestCase):
         self.assertEqual(socialaccount.user.username, "tutorials")
         self.assertEqual(socialaccount.user.email, "tutorials@bitbucket.org")
         account = socialaccount.get_provider_account()
-        self.assertEqual(account.to_str(), "tutorials account")
+        self.assertEqual(account.to_str(), "tutorials")
         self.assertEqual(account.get_profile_url(), "https://bitbucket.org/tutorials")
         self.assertEqual(
             account.get_avatar_url(),
             "https://bitbucket-assetroot.s3.amazonaws.com/c/photos/2013/Nov/25/tutorials-avatar-1563784409-6_avatar.png",  # noqa
         )
-        self.patches["requests"].get.assert_has_calls(
-            [
-                mock.call("https://api.bitbucket.org/2.0/user", params=mock.ANY),
-                mock.call(
-                    "https://api.bitbucket.org/2.0/user/emails",
-                    params=mock.ANY,
-                ),
-            ]
-        )
