@@ -1,12 +1,16 @@
-from allauth.socialaccount.tests import create_oauth2_tests
-from allauth.tests import MockedResponse
-from allauth.socialaccount.providers import registry
+from allauth.socialaccount.tests import OAuth2TestsMixin
+from allauth.tests import MockedResponse, TestCase
 
 from .provider import BitlyProvider
 
-class BitlyTests(create_oauth2_tests(registry.by_id(BitlyProvider.id))):
+
+class BitlyTests(OAuth2TestsMixin, TestCase):
+    provider_id = BitlyProvider.id
+
     def get_mocked_response(self):
-        return MockedResponse(200, """{
+        return MockedResponse(
+            200,
+            """{
             "data": {
                 "apiKey": "R_f6397a37e765574f2e198dba5bb59522",
                 "custom_short_domain": null,
@@ -22,4 +26,8 @@ class BitlyTests(create_oauth2_tests(registry.by_id(BitlyProvider.id))):
             },
             "status_code": 200,
             "status_txt": "OK"
-        }""")
+        }""",
+        )
+
+    def get_expected_to_str(self):
+        return "bitlyapioauthdemo"
