@@ -1,5 +1,4 @@
-import requests
-
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.base import ProviderException
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
@@ -7,11 +6,9 @@ from allauth.socialaccount.providers.oauth2.views import (
     OAuth2LoginView,
 )
 
-from .provider import DataportenProvider
 
-
-class DataportenAdapter(OAuth2Adapter):
-    provider_id = DataportenProvider.id
+class DataportenOAuth2Adapter(OAuth2Adapter):
+    provider_id = "dataporten"
     access_token_url = "https://auth.dataporten.no/oauth/token"
     authorize_url = "https://auth.dataporten.no/oauth/authorization"
     profile_url = "https://auth.dataporten.no/userinfo"
@@ -34,9 +31,13 @@ class DataportenAdapter(OAuth2Adapter):
 
         # Userinfo endpoint, for documentation see:
         # https://docs.dataporten.no/docs/oauth-authentication/
-        userinfo_response = requests.get(
-            self.profile_url,
-            headers=headers,
+        userinfo_response = (
+            get_adapter()
+            .get_requests_session()
+            .get(
+                self.profile_url,
+                headers=headers,
+            )
         )
         # Raise exception for 4xx and 5xx response codes
         userinfo_response.raise_for_status()
@@ -60,5 +61,5 @@ class DataportenAdapter(OAuth2Adapter):
         )
 
 
-oauth2_login = OAuth2LoginView.adapter_view(DataportenAdapter)
-oauth2_callback = OAuth2CallbackView.adapter_view(DataportenAdapter)
+oauth2_login = OAuth2LoginView.adapter_view(DataportenOAuth2Adapter)
+oauth2_callback = OAuth2CallbackView.adapter_view(DataportenOAuth2Adapter)
