@@ -1,6 +1,4 @@
-import requests
-
-from allauth.socialaccount.providers.discord.provider import DiscordProvider
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
     OAuth2CallbackView,
@@ -9,7 +7,7 @@ from allauth.socialaccount.providers.oauth2.views import (
 
 
 class DiscordOAuth2Adapter(OAuth2Adapter):
-    provider_id = DiscordProvider.id
+    provider_id = "discord"
     access_token_url = "https://discord.com/api/oauth2/token"
     authorize_url = "https://discord.com/api/oauth2/authorize"
     profile_url = "https://discord.com/api/users/@me"
@@ -19,7 +17,9 @@ class DiscordOAuth2Adapter(OAuth2Adapter):
             "Authorization": "Bearer {0}".format(token.token),
             "Content-Type": "application/json",
         }
-        extra_data = requests.get(self.profile_url, headers=headers)
+        extra_data = (
+            get_adapter().get_requests_session().get(self.profile_url, headers=headers)
+        )
 
         return self.get_provider().sociallogin_from_response(request, extra_data.json())
 
