@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import json
 
 from allauth.socialaccount.tests import OAuth2TestsMixin
@@ -27,11 +24,16 @@ class DropboxOAuth2Tests(OAuth2TestsMixin, TestCase):
                 "display_name": "All Auth",
                 "familiar_name": "All",
                 "given_name": "All",
-                "surname": "Auth"
+                "surname": "Auth",
             },
-            "profile_photo_url": ("https://dl-web.dropbox.com/account_photo"
-                                  "/get/dbid%ASDFasd3ASdfasdFAsd1AS2ASDF1aS"
-                                  "-DfAs?size=128x128"),
-            "referral_link": "https://db.tt/ASDfAsDf"
+            "profile_photo_url": (
+                "https://dl-web.dropbox.com/account_photo"
+                "/get/dbid%ASDFasd3ASdfasdFAsd1AS2ASDF1aS"
+                "-DfAs?size=128x128"
+            ),
+            "referral_link": "https://db.tt/ASDfAsDf",
         }
-        return [MockedResponse(200, json.dumps(payload))]
+        return MockedResponse(200, json.dumps(payload))
+
+    def get_expected_to_str(self):
+        return "allauth@example.com"
