@@ -1,20 +1,17 @@
 """Views for Eventbrite API v3."""
-import requests
 
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
     OAuth2CallbackView,
     OAuth2LoginView,
 )
 
-from .provider import EventbriteProvider
-
 
 class EventbriteOAuth2Adapter(OAuth2Adapter):
-
     """OAuth2Adapter for Eventbrite API v3."""
 
-    provider_id = EventbriteProvider.id
+    provider_id = "eventbrite"
 
     authorize_url = "https://www.eventbrite.com/oauth/authorize"
     access_token_url = "https://www.eventbrite.com/oauth/token"
@@ -22,7 +19,11 @@ class EventbriteOAuth2Adapter(OAuth2Adapter):
 
     def complete_login(self, request, app, token, **kwargs):
         """Complete login."""
-        resp = requests.get(self.profile_url, params={"token": token.token})
+        resp = (
+            get_adapter()
+            .get_requests_session()
+            .get(self.profile_url, params={"token": token.token})
+        )
         extra_data = resp.json()
         return self.get_provider().sociallogin_from_response(request, extra_data)
 
