@@ -1,6 +1,3 @@
-import json
-from unittest.mock import patch
-
 from django.contrib.auth import get_user_model
 from django.test.client import RequestFactory
 from django.test.utils import override_settings
@@ -10,7 +7,7 @@ from allauth.account import app_settings as account_settings
 from allauth.account.models import EmailAddress
 from allauth.socialaccount.models import SocialAccount
 from allauth.socialaccount.tests import OAuth2TestsMixin
-from allauth.tests import MockedResponse, TestCase
+from allauth.tests import MockedResponse, TestCase, mocked_response
 
 from .provider import FacebookProvider
 
@@ -54,6 +51,9 @@ class FacebookTests(OAuth2TestsMixin, TestCase):
             data = self.facebook_data
         return MockedResponse(200, data)
 
+    def get_expected_to_str(self):
+        return "raymond.penners"
+
     def test_username_conflict(self):
         User = get_user_model()
         User.objects.create(username="raymond.penners")
@@ -85,22 +85,33 @@ class FacebookTests(OAuth2TestsMixin, TestCase):
         script = self.provider.media_js(request)
         self.assertTrue('"appId": "app123id"' in script)
 
+    def test_token_auth(self):
+        with mocked_response(
+            {"access_token": "app_token"},
+            {
+                "data": {
+                    "app_id": "app123id",
+                    "is_valid": True,
+                }
+            },
+            self.get_mocked_response(),
+        ):
+            login = self.provider.verify_token(None, {"access_token": "dummy"})
+            assert login.user.email == "raymond.penners@example.com"
+            assert login.token.token == "dummy"
+
     def test_login_by_token(self):
         resp = self.client.get(reverse("account_login"))
-        with patch(
-            "allauth.socialaccount.providers.facebook.views.requests"
-        ) as requests_mock:
-            mocks = [
-                {"access_token": "app_token"},
-                {
-                    "data": {
-                        "app_id": "app123id",
-                        "is_valid": True,
-                    }
-                },
-                self.get_mocked_response().json(),
-            ]
-            requests_mock.get.return_value.json = lambda: mocks.pop(0)
+        with mocked_response(
+            {"access_token": "app_token"},
+            {
+                "data": {
+                    "app_id": "app123id",
+                    "is_valid": True,
+                }
+            },
+            self.get_mocked_response(),
+        ):
             resp = self.client.post(
                 reverse("facebook_login_by_token"),
                 data={"access_token": "dummy"},
@@ -120,22 +131,18 @@ class FacebookTests(OAuth2TestsMixin, TestCase):
     )
     def test_login_by_token_reauthenticate(self):
         resp = self.client.get(reverse("account_login"))
-        nonce = json.loads(resp.context["fb_data"])["loginOptions"]["auth_nonce"]
-        with patch(
-            "allauth.socialaccount.providers.facebook.views.requests"
-        ) as requests_mock:
-            mocks = [
-                {"access_token": "app_token"},
-                {
-                    "data": {
-                        "app_id": "app123id",
-                        "is_valid": True,
-                    }
-                },
-                {"auth_nonce": nonce},
-                self.get_mocked_response().json(),
-            ]
-            requests_mock.get.return_value.json = lambda: mocks.pop(0)
+        nonce = resp.context["fb_data"]["loginOptions"]["auth_nonce"]
+        with mocked_response(
+            {"access_token": "app_token"},
+            {
+                "data": {
+                    "app_id": "app123id",
+                    "is_valid": True,
+                }
+            },
+            {"auth_nonce": nonce},
+            self.get_mocked_response(),
+        ):
             resp = self.client.post(
                 reverse("facebook_login_by_token"),
                 data={"access_token": "dummy"},
