@@ -1,14 +1,18 @@
-from allauth.socialaccount.tests import create_oauth2_tests
-from allauth.tests import MockedResponse
-from allauth.socialaccount.providers import registry
+from allauth.socialaccount.tests import OAuth2TestsMixin
+from allauth.tests import MockedResponse, TestCase
 
 from .provider import HubicProvider
 
-class HubicTests(create_oauth2_tests(registry.by_id(HubicProvider.id))):
+
+class HubicTests(OAuth2TestsMixin, TestCase):
+    provider_id = HubicProvider.id
+
     def get_mocked_response(self):
-        return MockedResponse(200, """
+        return MockedResponse(
+            200,
+            """
 {
-    "email": "asdf@asdf.com",
+    "email": "user@example.com",
     "firstname": "Test",
     "activated": true,
     "creationDate": "2014-04-17T17:04:01+02:00",
@@ -17,7 +21,11 @@ class HubicTests(create_oauth2_tests(registry.by_id(HubicProvider.id))):
     "offer": "25g",
     "lastname": "User"
 }
-""")
+""",
+        )
+
+    def get_expected_to_str(self):
+        return "user@example.com"
 
     def get_login_response_json(self, with_refresh_token=True):
         return '{\
