@@ -8,12 +8,10 @@ class InstagramTests(OAuth2TestsMixin, TestCase):
     provider_id = InstagramProvider.id
 
     def get_mocked_response(self):
-        return MockedResponse(200, """
+        return MockedResponse(
+            200,
+            """
         {
-          "meta": {
-            "code": 200
-          },
-          "data": {
             "username": "georgewhewell",
             "bio": "",
             "website": "",
@@ -26,5 +24,8 @@ class InstagramTests(OAuth2TestsMixin, TestCase):
               "follows": 104
             },
             "id": "11428116"
-          }
-        }""")  # noqa
+        }""",
+        )  # noqa
+
+    def get_expected_to_str(self):
+        return "georgewhewell"
