@@ -15,7 +15,10 @@ class MailRuTests(OAuth2TestsMixin, TestCase):
         )  # noqa
 
     def get_login_response_json(self, with_refresh_token=True):
-        # FIXME: This is not an actual response. I added this in order
+        # TODO: This is not an actual response. I added this in order
         # to get the test suite going but did not verify to check the
         # exact response being returned.
         return '{"access_token": "testac", "uid": "weibo", "refresh_token": "testrf", "x_mailru_vid": "1"}'  # noqa
+
+    def get_expected_to_str(self):
+        return "emaslov@mail.ru"
