@@ -5,7 +5,6 @@ Inspired by:
     http://github.com/facebook/tornado/blob/master/tornado/auth.py
 """
 
-import requests
 from urllib.parse import parse_qsl, urlparse
 
 from django.http import HttpResponseRedirect
@@ -14,6 +13,7 @@ from django.utils.translation import gettext as _
 
 from requests_oauthlib import OAuth1
 
+from allauth.socialaccount.adapter import get_adapter
 from allauth.utils import build_absolute_uri, get_request_param
 
 
@@ -35,7 +35,7 @@ class OAuthError(Exception):
     pass
 
 
-class OAuthClient(object):
+class OAuthClient:
     def __init__(
         self,
         request,
@@ -78,7 +78,7 @@ class OAuthClient(object):
             )
             rt_url = self.request_token_url + "?" + urlencode(get_params)
             oauth = OAuth1(self.consumer_key, client_secret=self.consumer_secret)
-            response = requests.post(url=rt_url, auth=oauth)
+            response = get_adapter().get_requests_session().post(url=rt_url, auth=oauth)
             if response.status_code not in [200, 201]:
                 raise OAuthError(
                     _(
@@ -113,7 +113,7 @@ class OAuthClient(object):
             oauth_verifier = get_request_param(self.request, "oauth_verifier")
             if oauth_verifier:
                 at_url = at_url + "?" + urlencode({"oauth_verifier": oauth_verifier})
-            response = requests.post(url=at_url, auth=oauth)
+            response = get_adapter().get_requests_session().post(url=at_url, auth=oauth)
             if response.status_code not in [200, 201]:
                 raise OAuthError(
                     _("Invalid response while obtaining access token" ' from "%s".')
@@ -165,7 +165,7 @@ class OAuthClient(object):
         return HttpResponseRedirect(url)
 
 
-class OAuth(object):
+class OAuth:
     """
     Base class to perform oauth signed requests from access keys saved
     in a user's session. See the ``OAuthTwitter`` class below for an
@@ -204,7 +204,7 @@ class OAuth(object):
             resource_owner_key=access_token["oauth_token"],
             resource_owner_secret=access_token["oauth_token_secret"],
         )
-        response = getattr(requests, method.lower())(
+        response = getattr(get_adapter().get_requests_session(), method.lower())(
             url, auth=oauth, headers=headers, params=params
         )
         if response.status_code != 200:
