@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import absolute_import
-
 from allauth.socialaccount.tests import OAuth2TestsMixin
 from allauth.tests import MockedResponse, TestCase
 
@@ -24,5 +20,8 @@ class OdnoklassnikiTests(OAuth2TestsMixin, TestCase):
 """,
         )
 
+    def get_expected_to_str(self):
+        return "Ivan Petrov"
+
     def get_login_response_json(self, with_refresh_token=True):
         return '{"access_token": "testac"}'  # noqa
