@@ -1,7 +1,8 @@
-import requests
-
 from django.urls import reverse
 
+from allauth.account.internal.decorators import login_not_required
+from allauth.socialaccount.adapter import get_adapter
+from allauth.socialaccount.models import SocialToken
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
     OAuth2CallbackView,
@@ -10,9 +11,7 @@ from allauth.socialaccount.providers.oauth2.views import (
 from allauth.utils import build_absolute_uri
 
 
-class OpenIDConnectAdapter(OAuth2Adapter):
-    supports_state = True
-
+class OpenIDConnectOAuth2Adapter(OAuth2Adapter):
     def __init__(self, request, provider_id):
         self.provider_id = provider_id
         super().__init__(request)
@@ -21,7 +20,7 @@ class OpenIDConnectAdapter(OAuth2Adapter):
     def openid_config(self):
         if not hasattr(self, "_openid_config"):
             server_url = self.get_provider().server_url
-            resp = requests.get(server_url)
+            resp = get_adapter().get_requests_session().get(server_url)
             resp.raise_for_status()
             self._openid_config = resp.json()
         return self._openid_config
@@ -47,9 +46,11 @@ class OpenIDConnectAdapter(OAuth2Adapter):
     def profile_url(self):
         return self.openid_config["userinfo_endpoint"]
 
-    def complete_login(self, request, app, token, response):
-        response = requests.get(
-            self.profile_url, headers={"Authorization": "Bearer " + str(token)}
+    def complete_login(self, request, app, token: SocialToken, **kwargs):
+        response = (
+            get_adapter()
+            .get_requests_session()
+            .get(self.profile_url, headers={"Authorization": "Bearer " + token.token})
         )
         response.raise_for_status()
         extra_data = response.json()
@@ -63,11 +64,17 @@ class OpenIDConnectAdapter(OAuth2Adapter):
         return build_absolute_uri(request, callback_url, protocol)
 
 
+@login_not_required
 def login(request, provider_id):
-    view = OAuth2LoginView.adapter_view(OpenIDConnectAdapter(request, provider_id))
+    view = OAuth2LoginView.adapter_view(
+        OpenIDConnectOAuth2Adapter(request, provider_id)
+    )
     return view(request)
 
 
+@login_not_required
 def callback(request, provider_id):
-    view = OAuth2CallbackView.adapter_view(OpenIDConnectAdapter(request, provider_id))
+    view = OAuth2CallbackView.adapter_view(
+        OpenIDConnectOAuth2Adapter(request, provider_id)
+    )
     return view(request)
