@@ -1,5 +1,8 @@
 from allauth.socialaccount.providers.base import ProviderAccount
 from allauth.socialaccount.providers.oauth.provider import OAuthProvider
+from allauth.socialaccount.providers.openstreetmap.views import (
+    OpenStreetMapOAuthAdapter,
+)
 
 
 class OpenStreetMapAccount(ProviderAccount):
@@ -15,14 +18,12 @@ class OpenStreetMapAccount(ProviderAccount):
     def get_username(self):
         return self.account.extra_data["display_name"]
 
-    def to_str(self):
-        return self.get_username()
-
 
 class OpenStreetMapProvider(OAuthProvider):
     id = "openstreetmap"
     name = "OpenStreetMap"
     account_class = OpenStreetMapAccount
+    oauth_adapter_class = OpenStreetMapOAuthAdapter
 
     def extract_uid(self, data):
         return data["id"]
