@@ -1,16 +1,13 @@
-import requests
-
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
     OAuth2CallbackView,
     OAuth2LoginView,
 )
 
-from .provider import StackExchangeProvider
-
 
 class StackExchangeOAuth2Adapter(OAuth2Adapter):
-    provider_id = StackExchangeProvider.id
+    provider_id = "stackexchange"
     access_token_url = "https://stackexchange.com/oauth/access_token"
     authorize_url = "https://stackexchange.com/oauth"
     profile_url = "https://api.stackexchange.com/2.1/me"
@@ -18,9 +15,13 @@ class StackExchangeOAuth2Adapter(OAuth2Adapter):
     def complete_login(self, request, app, token, **kwargs):
         provider = self.get_provider()
         site = provider.get_site()
-        resp = requests.get(
-            self.profile_url,
-            params={"access_token": token.token, "key": app.key, "site": site},
+        resp = (
+            get_adapter()
+            .get_requests_session()
+            .get(
+                self.profile_url,
+                params={"access_token": token.token, "key": app.key, "site": site},
+            )
         )
         resp.raise_for_status()
         extra_data = resp.json()["items"][0]
