@@ -20,6 +20,9 @@ class TwentyThreeAndMeTests(OAuth2TestsMixin, TestCase):
         """,
         )
 
+    def get_expected_to_str(self):
+        return "23andMe"
+
     def get_login_response_json(self, with_refresh_token=True):
         return """
         {
