@@ -1,16 +1,13 @@
-import requests
-
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
     OAuth2CallbackView,
     OAuth2LoginView,
 )
 
-from .provider import TwitterOAuth2Provider
-
 
 class TwitterOAuth2Adapter(OAuth2Adapter):
-    provider_id = TwitterOAuth2Provider.id
+    provider_id = "twitter_oauth2"
     access_token_url = "https://api.twitter.com/2/oauth2/token"
     authorize_url = "https://twitter.com/i/oauth2/authorize"
     profile_url = "https://api.twitter.com/2/users/me"
@@ -26,10 +23,14 @@ class TwitterOAuth2Adapter(OAuth2Adapter):
         headers.update(self.get_provider().get_settings().get("HEADERS", {}))
         headers["Authorization"] = " ".join(["Bearer", token.token])
 
-        resp = requests.get(
-            url=self.profile_url,
-            params={"user.fields": ",".join(fields)},
-            headers=headers,
+        resp = (
+            get_adapter()
+            .get_requests_session()
+            .get(
+                url=self.profile_url,
+                params={"user.fields": ",".join(fields)},
+                headers=headers,
+            )
         )
         resp.raise_for_status()
         data = resp.json()["data"]
