@@ -3,6 +3,7 @@ from django.urls import reverse
 from allauth.account.models import EmailAddress
 from allauth.socialaccount.providers.base import ProviderAccount
 from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
+from allauth.socialaccount.providers.untappd.views import UntappdOAuth2Adapter
 
 
 class UntappdAccount(ProviderAccount):
@@ -12,18 +13,18 @@ class UntappdAccount(ProviderAccount):
     def get_avatar_url(self):
         return self.account.extra_data.get("user_avatar")
 
-    def to_str(self):
-        dflt = super(UntappdAccount, self).to_str()
-        return self.account.extra_data.get("user_name", dflt)
+    def get_user_data(self):
+        return self.account.extra_data.get("response", {}).get("user", {})
 
 
 class UntappdProvider(OAuth2Provider):
     id = "untappd"
     name = "Untappd"
     account_class = UntappdAccount
+    oauth2_adapter_class = UntappdOAuth2Adapter
 
-    def get_auth_params(self, request, action):
-        params = super(UntappdProvider, self).get_auth_params(request, action)
+    def get_auth_params_from_request(self, request, action):
+        params = super().get_auth_params_from_request(request, action)
         # Untappd uses redirect_url instead of redirect_uri
         params["redirect_url"] = request.build_absolute_uri(
             reverse(self.id + "_callback")
