@@ -28,3 +28,6 @@ class WeiboTests(OAuth2TestsMixin, TestCase):
 
 """,
         )
+
+    def get_expected_to_str(self):
+        return "pennersr"
