@@ -1,8 +1,8 @@
-import requests
 from collections import OrderedDict
 
 from django.utils.http import urlencode
 
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.oauth2.client import (
     OAuth2Client,
     OAuth2Error,
@@ -10,11 +10,12 @@ from allauth.socialaccount.providers.oauth2.client import (
 
 
 class WeixinOAuth2Client(OAuth2Client):
-    def get_redirect_url(self, authorization_url, extra_params):
+    def get_redirect_url(self, authorization_url, scope, extra_params):
+        scope = self.scope_delimiter.join(set(scope))
         params = {
             "appid": self.consumer_key,
             "redirect_uri": self.callback_url,
-            "scope": self.scope,
+            "scope": scope,
             "response_type": "code",
         }
         if self.state:
@@ -28,10 +29,8 @@ class WeixinOAuth2Client(OAuth2Client):
     def get_access_token(self, code, pkce_code_verifier=None):
         data = {
             "appid": self.consumer_key,
-            "redirect_uri": self.callback_url,
             "grant_type": "authorization_code",
             "secret": self.consumer_secret,
-            "scope": self.scope,
             "code": code,
         }
         params = None
@@ -43,7 +42,11 @@ class WeixinOAuth2Client(OAuth2Client):
         if data and pkce_code_verifier:
             data["code_verifier"] = pkce_code_verifier
         # TODO: Proper exception handling
-        resp = requests.request(self.access_token_method, url, params=params, data=data)
+        resp = (
+            get_adapter()
+            .get_requests_session()
+            .request(self.access_token_method, url, params=params, data=data)
+        )
         access_token = None
         if resp.status_code == 200:
             access_token = resp.json()
