@@ -1,15 +1,16 @@
-from allauth.socialaccount.tests import create_oauth2_tests
-from allauth.tests import MockedResponse
-from allauth.socialaccount.providers import registry
+from allauth.socialaccount.tests import OAuth2TestsMixin
+from allauth.tests import MockedResponse, TestCase
 
 from .provider import WindowsLiveProvider
 
 
-class WindowsLiveTests(create_oauth2_tests(
-        registry.by_id(WindowsLiveProvider.id))):
+class WindowsLiveTests(OAuth2TestsMixin, TestCase):
+    provider_id = WindowsLiveProvider.id
 
     def get_mocked_response(self):
-        return MockedResponse(200, """
+        return MockedResponse(
+            200,
+            """
         {
           "first_name": "James",
           "last_name": "Smith",
@@ -18,12 +19,16 @@ class WindowsLiveTests(create_oauth2_tests(
           "gender": null,
           "emails": {
               "personal": null,
-              "account": "jsmith@xyz.net",
+              "account": "jsmith@example.com",
               "business": null,
-              "preferred": "jsmith@xyz.net"
+              "preferred": "jsmith@example.com"
               },
           "link": "https://profile.live.com/",
           "updated_time": "2014-02-07T00:35:27+0000",
           "id": "83605e110af6ff98"
         }
-        """)
+        """,
+        )
+
+    def get_expected_to_str(self):
+        return "jsmith@example.com"
