@@ -1,27 +1,24 @@
-from __future__ import unicode_literals
-
-import requests
-
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
     OAuth2CallbackView,
     OAuth2LoginView,
 )
 
-from .provider import WindowsLiveProvider
-
 
 class WindowsLiveOAuth2Adapter(OAuth2Adapter):
-    provider_id = WindowsLiveProvider.id
+    provider_id = "windowslive"
     access_token_url = "https://login.live.com/oauth20_token.srf"
     authorize_url = "https://login.live.com/oauth20_authorize.srf"
     profile_url = "https://apis.live.net/v5.0/me"
 
     def complete_login(self, request, app, token, **kwargs):
         headers = {"Authorization": "Bearer {0}".format(token.token)}
-        resp = requests.get(self.profile_url, headers=headers)
+        resp = (
+            get_adapter().get_requests_session().get(self.profile_url, headers=headers)
+        )
 
-        # example of whats returned (in python format):
+        # example of what's returned (in python format):
         # {'first_name': 'James', 'last_name': 'Smith',
         #  'name': 'James Smith', 'locale': 'en_US', 'gender': None,
         #  'emails': {'personal': None, 'account': 'jsmith@example.com',
