@@ -10,10 +10,14 @@ from urllib.parse import urlsplit
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
-from django.core.exceptions import FieldDoesNotExist, ImproperlyConfigured
+from django.core.exceptions import (
+    FieldDoesNotExist,
+    ImproperlyConfigured,
+    ValidationError,
+)
 from django.core.files.base import ContentFile
 from django.core.serializers.json import DjangoJSONEncoder
-from django.core.validators import ValidationError, validate_email
+from django.core.validators import validate_email
 from django.db.models import FileField
 from django.db.models.fields import (
     BinaryField,
@@ -47,6 +51,8 @@ def _generate_unique_username_base(txts, regex=None):
             continue
         username = unicodedata.normalize("NFKD", force_str(txt))
         username = username.encode("ascii", "ignore").decode("ascii")
+        if len(username) == 0:
+            continue
         username = force_str(re.sub(regex, "", username).lower())
         # Django allows for '@' in usernames in order to accommodate for
         # project wanting to use email for username. In allauth we don't
@@ -127,7 +133,7 @@ def valid_email_or_none(email):
         if email:
             validate_email(email)
             if len(email) <= EmailField().max_length:
-                ret = email
+                ret = email.lower()
     except ValidationError:
         pass
     return ret
@@ -141,7 +147,7 @@ def import_attribute(path):
 
 
 def import_callable(path_or_callable):
-    if not hasattr(path_or_callable, "__call__"):
+    if not callable(path_or_callable):
         ret = import_attribute(path_or_callable)
     else:
         ret = path_or_callable
@@ -166,7 +172,8 @@ def serialize_instance(instance):
         try:
             field = instance._meta.get_field(k)
             if isinstance(field, BinaryField):
-                v = force_str(base64.b64encode(v))
+                if v is not None:
+                    v = force_str(base64.b64encode(v))
             elif isinstance(field, FileField):
                 if v and not isinstance(v, str):
                     v = {
